@@ -1,5 +1,28 @@
+#  ============================================================================
+#
+#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  www.conceptive.be / project-camelot@conceptive.be
+#
+#  This file is part of the Camelot Library.
+#
+#  This file may be used under the terms of the GNU General Public
+#  License version 2.0 as published by the Free Software Foundation
+#  and appearing in the file license.txt included in the packaging of
+#  this file.  Please review this information to ensure GNU
+#  General Public Licensing requirements will be met.
+#
+#  If you are unsure which license is appropriate for your use, please
+#  visit www.python-camelot.com or contact project-camelot@conceptive.be
+#
+#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
+#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+#
+#  For use of this library in commercial applications, please contact
+#  project-camelot@conceptive.be
+#
+#  ============================================================================
 from camelot.view.model_thread import model_function
-from camelot.core.utils import ugettext as _
+from camelot.core.utils import ugettext_lazy as _
 
 class Section(object):
     """A Section as displayed in the left pane of the application.  Each Section
@@ -23,7 +46,7 @@ class Section(object):
         return self.name
 
     def get_verbose_name(self):
-        return self.verbose_name or unicode(_(self.name)).capitalize()
+        return self.verbose_name or _(self.name.capitalize())
 
     def get_icon(self):
         from camelot.view.art import Icon
@@ -59,7 +82,7 @@ class SectionItem(object):
         self.verbose_name = verbose_name
 
     def get_verbose_name(self):
-        return self.verbose_name or unicode(_(self.action.get_verbose_name())).capitalize()
+        return self.verbose_name or self.action.get_verbose_name()
 
     def get_action(self):
         return self.action
@@ -72,3 +95,4 @@ def structure_to_section_items(structure):
         return SectionItem(element)
 
     return [rule(item) for item in structure]
+
