@@ -1,3 +1,26 @@
+#  ============================================================================
+#
+#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  www.conceptive.be / project-camelot@conceptive.be
+#
+#  This file is part of the Camelot Library.
+#
+#  This file may be used under the terms of the GNU General Public
+#  License version 2.0 as published by the Free Software Foundation
+#  and appearing in the file license.txt included in the packaging of
+#  this file.  Please review this information to ensure GNU
+#  General Public Licensing requirements will be met.
+#
+#  If you are unsure which license is appropriate for your use, please
+#  visit www.python-camelot.com or contact project-camelot@conceptive.be
+#
+#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
+#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+#
+#  For use of this library in commercial applications, please contact
+#  project-camelot@conceptive.be
+#
+#  ============================================================================
 '''
 Created on Aug 29, 2009
 
@@ -12,7 +35,7 @@ from sqlalchemy.orm.properties import PropertyLoader
 import pydot
 import types
 
-__all__ = ['create_uml_graph', 'create_schema_graph', 'show_uml_graph', 'show_schema_graph']
+__all__ = ['create_uml_graph', 'create_schema_graph',]
 
 def _mk_label(mapper, show_operations, show_attributes, show_datatypes, bordersize):
     html = '<<TABLE CELLSPACING="0" CELLPADDING="1" BORDER="0" CELLBORDER="%d" BALIGN="LEFT"><TR><TD><FONT POINT-SIZE="10">%s</FONT></TD></TR>' % (bordersize, mapper.class_.__name__)
@@ -173,3 +196,4 @@ def create_schema_graph(tables=None, metadata=None, show_indexes=True, show_data
 #                graph.edge_dst_list.append(fk.column.table.name)
 #            graph.sorted_graph_elements.append(graph_edge)
     return graph
+
