@@ -1,21 +1,18 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
 #
 #  This file may be used under the terms of the GNU General Public
 #  License version 2.0 as published by the Free Software Foundation
-#  and appearing in the file LICENSE.GPL included in the packaging of
-#  this file.  Please review the following information to ensure GNU
-#  General Public Licensing requirements will be met:
-#  http://www.trolltech.com/products/qt/opensource.html
+#  and appearing in the file license.txt included in the packaging of
+#  this file.  Please review this information to ensure GNU
+#  General Public Licensing requirements will be met.
 #
 #  If you are unsure which license is appropriate for your use, please
-#  review the following information:
-#  http://www.trolltech.com/products/qt/licensing.html or contact
-#  project-camelot@conceptive.be.
+#  visit www.python-camelot.com or contact project-camelot@conceptive.be
 #
 #  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 #  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -36,8 +33,11 @@ Those fields are stored in the :mod:`camelot.types` module.
 import logging
 logger = logging.getLogger('camelot.types')
 
+import sqlalchemy
 from sqlalchemy import types
 
+SQLALCHEMY_MAJOR_VERSION = int(sqlalchemy.__version__.split('.')[1])
+
 from camelot.core.files.storage import StoredFile, StoredImage, Storage
 
 class VirtualAddress(types.TypeDecorator):
@@ -49,7 +49,7 @@ class VirtualAddress(types.TypeDecorator):
     the :attr:`virtual_address_type`, and the second the address itself.
   
     eg: ``('email','project-camelot@conceptive.be')`` is stored as
-    ``mail://project-camelot@conceptive.be``
+    ``email://project-camelot@conceptive.be``
   
     .. image:: ../_static/virtualaddress_editor.png
     """
@@ -73,9 +73,12 @@ class VirtualAddress(types.TypeDecorator):
           
         return processor
     
-    def result_processor(self, dialect):
+    def result_processor(self, dialect, coltype=None):
       
-        impl_processor = self.impl.result_processor(dialect)
+        if SQLALCHEMY_MAJOR_VERSION >= 6:
+            impl_processor = self.impl.result_processor(dialect, coltype)
+        else:
+            impl_processor = self.impl.result_processor(dialect)
         if not impl_processor:
             impl_processor = lambda x:x
       
@@ -98,11 +101,13 @@ class Code(types.TypeDecorator):
     string joined with points.
   
     eg: ``['08', 'AB']`` is stored as ``08.AB``
+    
+    .. image:: ../_static/editors/CodeEditor_editable.png
     """
     
     impl = types.Unicode
           
-    def __init__(self, parts, **kwargs):
+    def __init__(self, parts, separator=u'.', **kwargs):
         """
         :param parts: a list of input masks specifying the mask for each part,
         eg ``['99', 'AA']``. For valid input masks, see
@@ -111,7 +116,8 @@ class Code(types.TypeDecorator):
         import string
         translator = string.maketrans('', '')
         self.parts = parts
-        max_length = sum(len(part.translate(translator, '<>!')) for part in parts) + len(parts)
+        self.separator = separator
+        max_length = sum(len(part.translate(translator, '<>!')) for part in parts) + len(parts)*len(self.separator)
         types.TypeDecorator.__init__(self, length=max_length, **kwargs)
         
     def bind_processor(self, dialect):
@@ -122,21 +128,24 @@ class Code(types.TypeDecorator):
           
         def processor(value):
             if value is not None:
-                value = u'.'.join(value)
+                value = self.separator.join(value)
             return impl_processor(value)
           
         return processor
     
-    def result_processor(self, dialect):
+    def result_processor(self, dialect, coltype=None):
       
-        impl_processor = self.impl.result_processor(dialect)
+        if SQLALCHEMY_MAJOR_VERSION >= 6:
+            impl_processor = self.impl.result_processor(dialect, coltype)
+        else:
+            impl_processor = self.impl.result_processor(dialect)
         if not impl_processor:
             impl_processor = lambda x:x
       
         def processor(value):
     
             if value:
-                return value.split('.')
+                return value.split(self.separator)
             return ['' for _p in self.parts]
             
         return processor
@@ -153,7 +162,8 @@ class Rating(types.TypeDecorator):
       title = Field(Unicode(60), required=True)
       rating = Field(camelot.types.Rating())
       
-  .. image:: ../_static/rating.png"""
+  .. image:: ../_static/editors/StarEditor_editable.png
+"""
     
     impl = types.Integer
        
@@ -161,25 +171,23 @@ class RichText(types.TypeDecorator):
     """RichText fields are unlimited text fields which contain html.  The html will be
   rendered in a rich text editor.  
   
-  .. image:: ../_static/richtext.png"""
+    .. image:: ../_static/editors/RichTextEditor_editable.png
+"""
     
     impl = types.UnicodeText
      
 class Language(types.TypeDecorator):
-    """The languages are stored as ISO codes in the database
+    """The languages are stored as a string in the database of 
+the form language[_country], where :
+
+ * language is a lowercase, two-letter, ISO 639 language code,
+ * territory is an uppercase, two-letter, ISO 3166 country code
+ 
+This used to be implemented using babel, but this was too slow and
+used too much memory, so now it's implemented using QT.
     """
     
-    import babel
-    
     impl = types.Unicode
-    choices = []
-    for code in babel.localedata.list():
-        locale = babel.Locale(code)
-        display_name = locale.get_display_name()
-        if display_name:
-            choices.append((code, u'%s (%s)'%(code, display_name)))
-        else:
-            choices.append((code, code))
     
     def __init__(self):
         types.TypeDecorator.__init__(self, length=20)
@@ -192,9 +200,8 @@ class Color(types.TypeDecorator):
   
     class MovieType(Entity):
       color = Field(camelot.types.Color())
-  
-  .. image:: ../_static/color.png
-  
+
+  .. image:: ../_static/editors/ColorEditor_editable.png  
   
   The colors are stored in the database as strings 
     """
@@ -221,9 +228,12 @@ class Color(types.TypeDecorator):
           
         return processor
       
-    def result_processor(self, dialect):
+    def result_processor(self, dialect, coltype=None):
       
-        impl_processor = self.impl.result_processor(dialect)
+        if SQLALCHEMY_MAJOR_VERSION >= 6:
+            impl_processor = self.impl.result_processor(dialect, coltype)
+        else:
+            impl_processor = self.impl.result_processor(dialect)
         if not impl_processor:
             impl_processor = lambda x:x
             
@@ -233,7 +243,7 @@ class Color(types.TypeDecorator):
                 return (int(value[2:4],16), int(value[4:6],16), int(value[6:8],16), int(value[0:2],16))
               
         return processor
-      
+        
 class Enumeration(types.TypeDecorator):
     """The enumeration field stores integers in the database, but represents them as
   strings.  This allows efficient storage and querying while preserving readable code.
@@ -248,7 +258,10 @@ class Enumeration(types.TypeDecorator):
       state = Field(camelot.types.Enumeration([(1,'planned'), (2,'recording'), (3,'finished'), (4,'canceled')]), 
                                               index=True, required=True, default='planning')
   
-  .. image:: ../_static/enumeration.png  
+  .. image:: ../_static/editors/ChoicesEditor_editable.png  
+  
+  If None should be a possible value of the enumeration, add (None, None) to the list of
+  possible enumerations.  None will be presented as empty in the GUI.
   """
     
     impl = types.Integer
@@ -276,12 +289,17 @@ class Enumeration(types.TypeDecorator):
                     return impl_processor(value)
                 except KeyError, e:
                     logger.error('could not process enumeration value %s, possible values are %s'%(value, u', '.join(list(self._string_to_int.keys()))), exc_info=e)
+            else:
+                impl_processor(value)
                         
         return processor
     
-    def result_processor(self, dialect):
+    def result_processor(self, dialect, coltype=None):
       
-        impl_processor = self.impl.result_processor(dialect)
+        if SQLALCHEMY_MAJOR_VERSION >= 6:
+            impl_processor = self.impl.result_processor(dialect, coltype)
+        else:
+            impl_processor = self.impl.result_processor(dialect)
         if not impl_processor:
             impl_processor = lambda x:x
             
@@ -299,13 +317,13 @@ class File(types.TypeDecorator):
     """Sqlalchemy column type to store files.  Only the location of the file is stored
     
   This column type accepts and returns a StoredFile, and stores them in the directory
-  specified by settings.MEDIA_ROOT.  The name of the file is stored as a string in
+  specified by settings.CAMELOT_MEDIA_ROOT.  The name of the file is stored as a string in
   the database.  A subdirectory upload_to can be specified::
   
     class Movie(Entity):
       script = Field(camelot.types.File(upload_to='script'))
       
-  .. image:: ../_static/file_delegate.png
+  .. image:: ../_static/editors/FileEditor_editable.png
     """
     
     impl = types.Unicode
@@ -330,9 +348,12 @@ class File(types.TypeDecorator):
           
         return processor
     
-    def result_processor(self, dialect):
+    def result_processor(self, dialect, coltype=None):
       
-        impl_processor = self.impl.result_processor(dialect)
+        if SQLALCHEMY_MAJOR_VERSION >= 6:
+            impl_processor = self.impl.result_processor(dialect, coltype)
+        else:
+            impl_processor = self.impl.result_processor(dialect)
         if not impl_processor:
             impl_processor = lambda x:x
             
@@ -354,7 +375,8 @@ class Image(File):
   The Image field type provides the same functionallity as the File field type, but
   the files stored should be images.
   
-  .. image:: ../_static/image.png  
+  .. image:: ../_static/editors/ImageEditor_editable.png
     """
   
     stored_file_implementation = StoredImage
+
