@@ -1,3 +1,26 @@
+#  ============================================================================
+#
+#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  www.conceptive.be / project-camelot@conceptive.be
+#
+#  This file is part of the Camelot Library.
+#
+#  This file may be used under the terms of the GNU General Public
+#  License version 2.0 as published by the Free Software Foundation
+#  and appearing in the file license.txt included in the packaging of
+#  this file.  Please review this information to ensure GNU
+#  General Public Licensing requirements will be met.
+#
+#  If you are unsure which license is appropriate for your use, please
+#  visit www.python-camelot.com or contact project-camelot@conceptive.be
+#
+#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
+#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+#
+#  For use of this library in commercial applications, please contact
+#  project-camelot@conceptive.be
+#
+#  ============================================================================
 from PyQt4 import QtGui, QtCore
 from PyQt4.QtCore import Qt
 
@@ -19,12 +42,6 @@ class SmileyDelegate(CustomDelegate):
                                 editable=editable,
                                 maximum=1,
                                 **kwargs)
-        maximum = 1
-        
-        self.maximum = maximum
-        
-        
-    
         
     def paint(self, painter, option, index):
         painter.save()
@@ -49,3 +66,4 @@ class SmileyDelegate(CustomDelegate):
         QtGui.QApplication.style().drawItemPixmap(painter, rect, 1, icon)
         rect = QtCore.QRect(rect.left()+20, rect.top(), rect.width(), rect.height())
         painter.restore()
+
