@@ -1,21 +1,18 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
 #
 #  This file may be used under the terms of the GNU General Public
 #  License version 2.0 as published by the Free Software Foundation
-#  and appearing in the file LICENSE.GPL included in the packaging of
-#  this file.  Please review the following information to ensure GNU
-#  General Public Licensing requirements will be met:
-#  http://www.trolltech.com/products/qt/opensource.html
+#  and appearing in the file license.txt included in the packaging of
+#  this file.  Please review this information to ensure GNU
+#  General Public Licensing requirements will be met.
 #
 #  If you are unsure which license is appropriate for your use, please
-#  review the following information:
-#  http://www.trolltech.com/products/qt/licensing.html or contact
-#  project-camelot@conceptive.be.
+#  visit www.python-camelot.com or contact project-camelot@conceptive.be
 #
 #  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 #  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -32,17 +29,72 @@ import sqlalchemy.sql.expression
 
 from camelot.admin.object_admin import ObjectAdmin
 from camelot.view.model_thread import post, model_function, gui_function
-from camelot.core.utils import ugettext as _
+from camelot.core.utils import ugettext_lazy, ugettext
 from camelot.admin.validator.entity_validator import EntityValidator
 
 
 class EntityAdmin(ObjectAdmin):
     """Admin class specific for classes that are mapped by sqlalchemy.
-    This allows for much more introspection than the standard ObjectAdmin.
+This allows for much more introspection than the standard ObjectAdmin.
+    
+It has additional class attributes that customise its behaviour.
+
+**Filtering**
+
+.. attribute:: list_filter
+
+A list of fields that should be used to generate filters for in the table
+view.  If the field named is a one2many, many2one or many2many field, the
+field name should be followed by a field name of the related entity ::
+
+    class Project(Entity):
+      oranization = OneToMany('Organization')
+      name = Field(Unicode(50))
+    
+      class Admin(EntityAdmin):
+        list_display = ['organization']
+        list_filter = ['organization.name']
+
+.. image:: ../_static/filter/group_box_filter.png
+
+
+**Searching**
+
+.. attribute:: list_search
+
+A list of fields that should be searched when the user enters something in
+the search box in the table view.  By default all fields are
+searched for which Camelot can do a conversion of the entered string to the
+datatype of the underlying column.  
+
+For use with one2many, many2one or many2many fields, the same rules as for the 
+list_filter attribute apply
+
+.. attribute:: search_all_fields
+
+Defaults to True, meaning that by default all searchable fields should be
+searched.  If this is set to False, one should explicitely set the list_search
+attribute to enable search.
+ 
     """
 
+    list_search = []
+    search_all_fields = True
     validator = EntityValidator
 
+    def __init__(self, app_admin, entity):
+        super(EntityAdmin, self).__init__(app_admin, entity)
+        from sqlalchemy import orm
+        from sqlalchemy.orm.exc import UnmappedClassError
+        from sqlalchemy.orm.mapper import _mapper_registry
+        try:
+            self.mapper = orm.class_mapper(self.entity)
+        except UnmappedClassError, exception:
+            mapped_entities = [unicode(m) for m in _mapper_registry.keys()]
+            logger.error(u'%s is not a mapped class, configured mappers include %s'%(self.entity, u','.join(mapped_entities)),
+                         exc_info=exception)
+            raise exception
+
     @model_function
     def get_query(self):
         """:return: an sqlalchemy query for all the objects that should be
@@ -51,50 +103,24 @@ class EntityAdmin(ObjectAdmin):
         """
         return self.entity.query
 
-    @model_function
-    def get_subclass_entity_admin(self, entity):
-        """Get the admin class for an entity that is a subclass of this admin's
-        entity or this admin's entity itself.
-        """
-        for subclass_admin in self.get_subclasses():
-            if subclass_admin.entity == entity:
-                return subclass_admin
-        return self
-
-    @model_function
-    def get_subclasses(self):
-        """Returns admin objects for the subclasses of the Entity represented
-        by this admin object.
-        """
-        if not self._subclasses:
-            from elixir import entities
-            self._subclasses = [
-                e.Admin(self.app_admin, e)
-                for e in entities
-                    if (
-                        issubclass(e, (self.entity,))
-                        and hasattr(e, 'Admin')
-                        and e != self.entity
-                    )
-            ]
-        return self._subclasses
-
     @model_function
     def get_verbose_identifier(self, obj):
-        if hasattr(obj, 'id') and obj.id:
-            if hasattr(obj, '__unicode__'):
-                return u'%s %s : %s' % (
-                    unicode(self.get_verbose_name().capitalize()),
-                    unicode(obj.id),
-                    unicode(obj)
-                )
-            else:
-                return u'%s %s' % (
-                    self.get_verbose_name().capitalize(),
-                    unicode(obj.id)
-                )
-        else:
-            return self.get_verbose_name()
+        if obj:
+            primary_key = self.mapper.primary_key_from_instance(obj)
+            if not None in primary_key:
+                primary_key_representation = u','.join([unicode(v) for v in primary_key])
+                if hasattr(obj, '__unicode__'):
+                    return u'%s %s : %s' % (
+                        unicode(self.get_verbose_name()),
+                        primary_key_representation,
+                        unicode(obj)
+                    )
+                else:
+                    return u'%s %s' % (
+                        self.get_verbose_name(),
+                        primary_key_representation
+                    )
+        return self.get_verbose_name()
 
     @model_function
     def get_field_attributes(self, field_name):
@@ -108,27 +134,32 @@ class EntityAdmin(ObjectAdmin):
          * widget : which widget to be used to render the field
          * ...
         """
-
         try:
             return self._field_attributes[field_name]
         except KeyError:
+
+            def create_default_getter(field_name):
+                return lambda o:getattr(o, field_name)
+
             from camelot.view.controls import delegates
             #
             # Default attributes for all fields
             #
             attributes = dict(
                 python_type = str,
+                field_name = field_name,
+                getter = create_default_getter(field_name),
                 length = None,
                 tooltip = None,
                 background_color = None,
-                minimal_column_width = 0,
+                minimal_column_width = 12,
                 editable = False,
                 nullable = True,
                 widget = 'str',
                 blank = True,
                 delegate = delegates.PlainTextDelegate,
                 validator_list = [],
-                name = unicode(_(field_name.replace('_', ' '))).capitalize()
+                name = ugettext_lazy(field_name.replace('_', ' ').capitalize())
             )
 
             #
@@ -161,20 +192,20 @@ class EntityAdmin(ObjectAdmin):
             from sqlalchemy import orm
             from sqlalchemy.exceptions import InvalidRequestError
             from field_attributes import _sqlalchemy_to_python_type_
-            mapper = orm.class_mapper(self.entity)
+
             try:
-                property = mapper.get_property(
+                property = self.mapper.get_property(
                     field_name,
                     resolve_synonyms=True
                 )
                 if isinstance(property, orm.properties.ColumnProperty):
-                    type = property.columns[0].type
+                    column_type = property.columns[0].type
                     python_type = _sqlalchemy_to_python_type_.get(
-                        type.__class__,
+                        column_type.__class__,
                         None
                     )
                     if python_type:
-                        attributes.update(python_type(type))
+                        attributes.update(python_type(column_type))
                     if not isinstance(
                         property.columns[0],
                         sqlalchemy.sql.expression._Label
@@ -183,7 +214,16 @@ class EntityAdmin(ObjectAdmin):
                         attributes['default'] = property.columns[0].default
                 elif isinstance(property, orm.properties.PropertyLoader):
                     target = property._get_target().class_
-                    foreign_keys = list(property._foreign_keys)
+                    
+                    #
+                    # _foreign_keys is for sqla pre 0.6.4
+                    # 
+                    if hasattr(property, '_foreign_keys'):
+                        foreign_keys = list(property._foreign_keys)
+                    else:
+                        foreign_keys = list( property._user_defined_foreign_keys )
+                        foreign_keys.extend( list(property._calculated_foreign_keys) )
+                        
                     if property.direction == orm.interfaces.ONETOMANY:
                         attributes.update(
                             python_type = list,
@@ -192,7 +232,6 @@ class EntityAdmin(ObjectAdmin):
                             delegate = delegates.One2ManyDelegate,
                             target = target,
                             create_inline = False,
-                            backref = property.backref.key,
                             direction = property.direction,
                             admin = get_entity_admin(target)
                         )
@@ -203,7 +242,7 @@ class EntityAdmin(ObjectAdmin):
                             delegate = delegates.Many2OneDelegate,
                             target = target,
                             #
-                            #@todo: take into account all foreign keys instead
+                            # @todo: take into account all foreign keys instead
                             # of only the first one
                             #
                             nullable = foreign_keys[0].nullable,
@@ -245,13 +284,26 @@ class EntityAdmin(ObjectAdmin):
             #
             if 'target' in attributes:
                 attributes['admin'] = get_entity_admin(attributes['target'])
-
-            # if name should be translated, do so now
-            attributes['name'] = unicode(attributes['name']).capitalize()
-
+            
             self._field_attributes[field_name] = attributes
             return attributes
 
+    def get_dynamic_field_attributes(self, obj, field_names):
+        """Takes the dynamic field attributes from through the ObjectAdmin its
+        get_dynamic_field_attributes and add the new_message attributes for
+        One2Many fields where the object was not flushed yet
+        """
+        from sqlalchemy.orm.session import Session
+        session = Session.object_session( obj )
+        # when a previous flush failed, obj might have no session
+        if session and obj in session.new:
+            new_message = ugettext('Please complete the form first')
+        else:
+            new_message = None
+        for attributes in super(EntityAdmin, self).get_dynamic_field_attributes(obj, field_names):
+            attributes['new_message'] = new_message
+            yield attributes
+            
     @model_function
     def get_list_charts(self):
         return self.list_charts
@@ -272,12 +324,28 @@ class EntityAdmin(ObjectAdmin):
         return list(filter_generator())
 
     @model_function
-    def set_defaults(self, entity_instance):
-        """Set the defaults of an object"""
+    def set_defaults(self, entity_instance, include_nullable_fields=True):
+        """Set the defaults of an object
+        :param include_nullable_fields: also set defaults for nullable fields, depending
+        on the context, this should be set to False to allow the user to set the field
+        to None
+        """
         from sqlalchemy.schema import ColumnDefault
         for field, attributes in self.get_fields():
+            has_default = False
             try:
                 default = attributes['default']
+                has_default = True
+            except KeyError:
+                pass
+            if has_default:
+                #
+                # prevent the setting of a default value when one has been
+                # set allready
+                #
+                value = attributes['getter'](entity_instance)
+                if value!=None: # False is a legitimate value for Booleans
+                    continue
                 if isinstance(default, ColumnDefault):
                     default_value = default.execute()
                 elif callable(default):
@@ -298,7 +366,7 @@ class EntityAdmin(ObjectAdmin):
                 )
                 try:
                     setattr(entity_instance, field, default_value)
-                except AttributeError, e:
+                except AttributeError, exc:
                     logger.error(
                         'Programming Error : could not set'
                         ' attribute %s to %s on %s' % (
@@ -306,167 +374,12 @@ class EntityAdmin(ObjectAdmin):
                             default_value,
                             entity_instance.__class__.__name__
                         ),
-                        exc_info=e
+                        exc_info=exc
                     )
-            except KeyError, _e:
-                pass
+
 
     @gui_function
-    def create_new_view(admin, parent=None, oncreate=None, onexpunge=None):
-        """Create a Qt widget containing a form to create a new instance of the
-        entity related to this admin class
-
-        The returned class has an 'entity_created_signal' that will be fired
-        when a valid new entity was created by the form
-        """
-        from PyQt4 import QtCore
-        from PyQt4 import QtGui
-        from PyQt4.QtCore import SIGNAL
-        from camelot.view.controls.view import AbstractView
-        from proxy.collection_proxy import CollectionProxy
-        new_object = []
-
-        @model_function
-        def collection_getter():
-            if not new_object:
-                entity_instance = admin.entity()
-                if oncreate:
-                    oncreate(entity_instance)
-                # Give the default fields their value
-                admin.set_defaults(entity_instance)
-                new_object.append(entity_instance)
-            return new_object
-
-        model = CollectionProxy(
-            admin,
-            collection_getter,
-            admin.get_fields,
-            max_number_of_rows=1
-        )
-        validator = admin.create_validator(model)
-
-        class NewForm(AbstractView):
-
-            def __init__(self, parent):
-                AbstractView.__init__(self, parent)
-                self.widget_layout = QtGui.QVBoxLayout()
-                self.widget_layout.setMargin(0)
-                title = _('new')
-                index = 0
-                self.form_view = admin.create_form_view(
-                    title, model, index, parent
-                )
-                self.widget_layout.insertWidget(0, self.form_view)
-                self.setLayout(self.widget_layout)
-                self.validate_before_close = True
-                self.entity_created_signal = SIGNAL('entity_created')
-                #
-                # every time data has been changed, it could become valid,
-                # when this is the case, it should be propagated
-                #
-                self.connect(
-                    model,
-                    SIGNAL(
-                        'dataChanged(const QModelIndex &, const QModelIndex &)'
-                    ),
-                    self.dataChanged
-                )
-                self.connect(
-                    self.form_view,
-                    AbstractView.title_changed_signal,
-                    self.change_title
-                )
-
-            def emit_if_valid(self, valid):
-                if valid:
-
-                    def create_instance_getter(new_object):
-                        return lambda:new_object[0]
-
-                    self.emit(
-                        self.entity_created_signal,
-                        create_instance_getter(new_object)
-                    )
-
-            def dataChanged(self, index1, index2):
-
-                def validate():
-                    return validator.isValid(0)
-
-                post(validate, self.emit_if_valid)
-
-            def showMessage(self, valid):
-                from camelot.view.workspace import get_workspace
-                if not valid:
-                    row = 0
-                    reply = validator.validityDialog(row, self).exec_()
-                    if reply == QtGui.QMessageBox.Discard:
-                        # clear mapping to prevent data being written again to
-                        # the model, after we reverted the row
-                        self.form_view.widget_mapper.clearMapping()
-
-                        def onexpunge_on_all():
-                            if onexpunge:
-                                for o in new_object:
-                                    onexpunge(o)
-
-                        post(onexpunge_on_all)
-                        self.validate_before_close = False
-
-                        for window in get_workspace().subWindowList():
-                            if window.widget() == self:
-                                window.close()
-                else:
-                    def create_instance_getter(new_object):
-                        return lambda:new_object[0]
-
-                    for _o in new_object:
-                        self.emit(
-                            self.entity_created_signal,
-                            create_instance_getter(new_object)
-                        )
-                    self.validate_before_close = False
-                    from camelot.view.workspace import NoDesktopWorkspace
-                    workspace = get_workspace()
-                    if isinstance(workspace, (NoDesktopWorkspace,)):
-                        self.close()
-                    else:
-                        for window in get_workspace().subWindowList():
-                            if window.widget() == self:
-                                window.close()
-
-            def validateClose(self):
-                logger.debug(
-                    'validate before close : %s' %
-                    self.validate_before_close
-                )
-                if self.validate_before_close:
-                    self.form_view.widget_mapper.submit()
-                    logger.debug(
-                        'unflushed rows : %s' %
-                        str(model.hasUnflushedRows())
-                    )
-                    if model.hasUnflushedRows():
-                        def validate(): return validator.isValid(0)
-                        post(validate, self.showMessage)
-                        return False
-                    else:
-                        return True
-                return True
-
-            def closeEvent(self, event):
-                if self.validateClose():
-                    event.accept()
-                else:
-                    event.ignore()
-
-        form = NewForm(parent)
-        if hasattr(admin, 'form_size'):
-            form.setMinimumSize(admin.form_size[0], admin.form_size[1])
-        return form
-
-    @gui_function
-    def create_select_view(admin, query, search_text=None, parent=None):
+    def create_select_view(admin, query=None, search_text=None, parent=None):
         """Returns a Qt widget that can be used to select an element from a
         query
 
@@ -476,37 +389,30 @@ class EntityAdmin(ObjectAdmin):
         returned widget has an entity_selected_signal signal that will be fired
         when a entity has been selected.
         """
-        from controls.tableview import TableView
         from art import Icon
         from proxy.queryproxy import QueryTableProxy
-        from PyQt4.QtCore import SIGNAL
+        from PyQt4 import QtCore
 
         class SelectQueryTableProxy(QueryTableProxy):
-            header_icon = Icon(
-                'tango/16x16/emblems/emblem-symbolic-link.png'
-            ).getQIcon()
+            header_icon = Icon('tango/16x16/emblems/emblem-symbolic-link.png')
 
-        class SelectView(TableView):
-
-            query_table_proxy = SelectQueryTableProxy
+        class SelectView(admin.TableView):
+            table_model = SelectQueryTableProxy
+            entity_selected_signal = QtCore.pyqtSignal(object)
             title_format = 'Select %s'
 
             def __init__(self, admin, parent):
-                TableView.__init__(
-                    self, admin,
+                super(SelectView, self).__init__(
+                    admin,
                     search_text=search_text, parent=parent
                 )
-                self.entity_selected_signal = SIGNAL('entity_selected')
-                self.connect(self, SIGNAL('row_selected'), self.sectionClicked)
+                self.row_selected_signal.connect( self.sectionClicked )
                 self.setUpdatesEnabled(True)
 
-            def emit_and_close(self, instance_getter):
-                self.emit(self.entity_selected_signal, instance_getter)
-                from camelot.view.workspace import get_workspace
-                for window in get_workspace().subWindowList():
-                    if hasattr(window, 'widget') and window.widget() == self:
-                        window.close()
+            def emit_entity_selected(self, instance_getter):
+                self.entity_selected_signal.emit( instance_getter )
 
+            @QtCore.pyqtSlot(int)
             def sectionClicked(self, index):
                 # table model will be set by the model thread, we can't
                 # decently select if it has not been set yet
@@ -519,7 +425,7 @@ class EntityAdmin(ObjectAdmin):
                         entity = self._table_model._get_object(index)
                         return create_constant_getter(entity)
 
-                    post(create_instance_getter, self.emit_and_close)
+                    post(create_instance_getter, self.emit_entity_selected)
 
         widget = SelectView(admin, parent)
         widget.setUpdatesEnabled(True)
@@ -535,61 +441,110 @@ class EntityAdmin(ObjectAdmin):
 
         :param query_getter: sqlalchemy query object
 
-        :param parent: the workspace widget that will contain the table view
+        :param parent: the widget that will contain the table view
         """
+        from camelot.view.workspace import show_top_level
 
-        from PyQt4 import QtCore
-        from PyQt4.QtCore import SIGNAL
-        from controls.tableview import TableView
         from proxy.queryproxy import QueryTableProxy
-        tableview = TableView(self)
+        tableview = self.TableView(self)
 
         def createOpenForm(self, tableview):
 
             def openForm(index):
-                from workspace import get_workspace
                 model = QueryTableProxy(
                     tableview.admin,
-                    tableview._table_model._query_getter,
+                    tableview._table_model.get_query_getter(),
                     tableview.admin.get_fields,
                     max_number_of_rows=1
                 )
                 title = ''
                 formview = tableview.admin.create_form_view(
-                    title, model, index, parent
+                    title, model, index, parent=None
                 )
-                get_workspace().addSubWindow(formview)
-                formview.show()
+                show_top_level( formview, tableview )
+                # @todo: dirty trick to keep reference
+                #self.__form = formview
 
             return openForm
 
-        tableview.connect(
-            tableview,
-            SIGNAL('row_selected'),
+        tableview.row_selected_signal.connect(
             createOpenForm(self, tableview)
         )
 
         return tableview
-    
+
     @model_function
     def delete(self, entity_instance):
         """Delete an entity instance"""
         from sqlalchemy.orm.session import Session
-        if hasattr(entity_instance, 'id') and entity_instance.id:
-                pk = entity_instance.id
-                # save the state before the update
-                from camelot.model.memento import BeforeDelete
-                from camelot.model.authentication import getCurrentAuthentication
-                history = BeforeDelete( model = unicode( self.entity.__name__ ),
-                                       primary_key = pk,
-                                       previous_attributes = {},
-                                       authentication = getCurrentAuthentication() )
+        session = Session.object_session( entity_instance )
+        #
+        # new and deleted instances cannot be deleted
+        #
+        if session:
+            if entity_instance in session.new:
+                session.expunge(entity_instance)
+            elif (entity_instance not in session.deleted) and \
+                 (entity_instance in session): # if the object is not in the session, it might allready be deleted
+                history = None
+                #
+                # only if we know the primary key, we can keep track of its history
+                #
+                primary_key = self.mapper.primary_key_from_instance(entity_instance)
+                #
+                # we can only store history of objects where the primary key has only
+                # 1 element
+                # @todo: store history for compound primary keys
+                #
+                if not None in primary_key and len(primary_key)==1:
+                    pk = primary_key[0]
+                    # save the state before the update
+                    from camelot.model.memento import BeforeDelete
+                    # only register the delete when the camelot model is active
+                    if hasattr(BeforeDelete, 'query'):
+                        from camelot.model.authentication import getCurrentAuthentication
+                        history = BeforeDelete( model = unicode( self.entity.__name__ ),
+                                                primary_key = pk,
+                                                previous_attributes = {},
+                                                authentication = getCurrentAuthentication() )
                 entity_instance.delete()
-                Session.object_session( entity_instance ).flush( [entity_instance] )
-                Session.object_session( history ).flush( [history] )
+                session.flush( [entity_instance] )
+                if history:
+                    Session.object_session( history ).flush( [history] )
 
     @model_function
     def flush(self, entity_instance):
         """Flush the pending changes of this entity instance to the backend"""
         from sqlalchemy.orm.session import Session
-        Session.object_session( entity_instance ).flush( [entity_instance] )
+        session = Session.object_session( entity_instance )
+        if not session:
+            logger.error('Programming Error : entity %s cannot be flushed because it has no session'%(unicode(entity_instance)))
+        else:
+            session.flush( [entity_instance] )
+
+    @model_function
+    def refresh(self, entity_instance):
+        """Undo the pending changes to the backend and restore the original
+        state"""
+        from sqlalchemy.orm.session import Session
+        session = Session.object_session( entity_instance )
+        if session:
+            session.refresh( entity_instance )
+
+    @model_function
+    def copy(self, entity_instance):
+        """Duplicate this entity instance"""
+        from sqlalchemy import orm
+        new_entity_instance = entity_instance.__class__()
+        new_entity_instance.from_dict( entity_instance.to_dict(exclude=[c.name for c in self.mapper.primary_key]) )
+        #
+        # recreate the ManyToOne relations
+        #
+        for property in self.mapper.iterate_properties:
+            if isinstance(property, orm.properties.PropertyLoader):
+                if property.direction == orm.interfaces.MANYTOONE:
+                    setattr( new_entity_instance, 
+                             property.key,
+                             getattr( entity_instance, property.key ) )
+        return new_entity_instance
+
