@@ -1,21 +1,18 @@
-#  ==================================================================================
+#  ============================================================================
 #
-#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
 #
 #  This file may be used under the terms of the GNU General Public
 #  License version 2.0 as published by the Free Software Foundation
-#  and appearing in the file LICENSE.GPL included in the packaging of
-#  this file.  Please review the following information to ensure GNU
-#  General Public Licensing requirements will be met:
-#  http://www.trolltech.com/products/qt/opensource.html
+#  and appearing in the file license.txt included in the packaging of
+#  this file.  Please review this information to ensure GNU
+#  General Public Licensing requirements will be met.
 #
 #  If you are unsure which license is appropriate for your use, please
-#  review the following information:
-#  http://www.trolltech.com/products/qt/licensing.html or contact
-#  project-camelot@conceptive.be.
+#  visit www.python-camelot.com or contact project-camelot@conceptive.be
 #
 #  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 #  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -23,29 +20,53 @@
 #  For use of this library in commercial applications, please contact
 #  project-camelot@conceptive.be
 #
-#  ==================================================================================
+#  ============================================================================
 
 import logging
-logger = logging.getLogger('camelot.view.export.excel')
-
-from pyExcelerator import *
 import datetime
+import settings
+LOGGER = logging.getLogger('camelot.view.export.excel')
+
+from camelot.view.controls import delegates
+from camelot.view.utils import local_date_format
+
+# previously used pyExcelerator, but this gave errors opening the generated documents in Excel 2010
+from xlwt import Font, Borders, XFStyle, Pattern, Workbook, ExcelFormula
 
 titleFont = Font()              # initializing titleFont Object
 headerFont = Font()             # initializing headerFont Object
 cellFont = Font()               # initializing cellFont Object
 
-titleFont.name = 'Arial'        # Setting Fonts Name
-headerFont.name = 'Arial'
-cellFont.name = 'Arial'
+# when you need a specific font or size, declare this in your settings file
+fontname = 'Arial'
+if hasattr(settings, 'EXPORT_EXCEL_FONT'):
+    fontname = settings.EXPORT_EXCEL_FONT
+titleFont.name = fontname        # Setting Fonts Name
+headerFont.name = fontname
+cellFont.name = fontname
 
-titleFont.bold = True           # Setting title font to bold
-headerFont.bold = True          # Setting column header font to bold
-cellFont.bold = False           # Setting cell font to bold
+title_bold = True
+header_bold = True
+cell_bold = False
+if hasattr(settings, 'EXPORT_EXCEL_TITLE_BOLD'):
+    title_bold = settings.EXPORT_EXCEL_TITLE_BOLD
+if hasattr(settings, 'EXPORT_EXCEL_HEADER_BOLD'):
+    header_bold = settings.EXPORT_EXCEL_HEADER_BOLD
+if hasattr(settings, 'EXPORT_EXCEL_CELL_BOLD'):
+    cell_bold = settings.EXPORT_EXCEL_CELL_BOLD
+titleFont.bold = title_bold           # Setting title font to bold
+headerFont.bold = header_bold          # Setting column header font to bold
+cellFont.bold = cell_bold           # Setting cell font to bold
 
-titleFont.height = 240          # 12*20 = 240 Font Size
-headerFont.height = 220         # 10*20 = 240 Font Size
-cellFont.height = 220           # 10*20 = 240 Font Size
+titlesize = 240 # 12*20 = 240 Font Size
+fontsize = 200# 10*20 = 200 Font Size
+if hasattr(settings, 'EXPORT_EXCEL_TITLE_SIZE'):
+    titlesize = settings.EXPORT_EXCEL_TITLE_SIZE
+if hasattr(settings, 'EXPORT_EXCEL_FONT_SIZE'):
+    fontsize = settings.EXPORT_EXCEL_FONT_SIZE
+titleFont.height = titlesize          
+headerFont.height = fontsize         
+cellFont.height = fontsize
 
 brdLeft = Borders()                # Defining border which is around header
 brdLeft.left = 0x01
@@ -140,13 +161,14 @@ def open_data_with_excel(title, headerList, dataList):
         excel_app.Visible = True
         excel_app.Workbooks.Open(xls_fn)
 
-def write_data_to_excel(filename, title, headerList, dataList):
+def write_data_to_excel(filename, title, headerList, data_list):
     """
     @param filename: the file to which to save the exported excel
     @param title: title to put in the first row of the genarated excel file
     @param headerList: list of header definitions
-    @param dataList: list or generator with the row data
+    @param data_list: list or generator with the row data
     """
+    LOGGER.debug(u'write data to excel : %s'%title)
     w = Workbook()
     ws = w.add_sheet('Sheet1')
     ## Writing Title
@@ -157,38 +179,45 @@ def write_data_to_excel(filename, title, headerList, dataList):
     myPrecisionDict = {}        # dictionary of precision , {columnnumber, Precision}
     myLengthDict = {}           # dictionary of length , {columnnumber, length}
     myFormatDict = {}           # dictionary of dateformat , {columnnumber, format}
+    header_delegates = {}
+    number_of_columns = len(headerList)
     for n,desc in enumerate(headerList):
         lst =  desc[1]
+        header_delegates[n] = lst['delegate']
         if n==0:
-            ws.write(2, n, lst['name'], topleftCellStyle)
+            ws.write(2, n, unicode(lst['name']), topleftCellStyle)
         elif n==len(headerList)-1:
-            ws.write(2, n, lst['name'], toprightCellStyle)
+            ws.write(2, n, unicode(lst['name']), toprightCellStyle)
         else:
-            ws.write(2, n, lst['name'], headerStyle)
-        if len(lst['name']) < 8:
+            ws.write(2, n, unicode(lst['name']), headerStyle)
+        if len(unicode(lst['name'])) < 8:
             ws.col(n).width = 8 *  375
         else:
-            ws.col(n).width = len(lst['name']) *  375
+            ws.col(n).width = len(unicode(lst['name'])) *  375
         myDataTypeDict[ n ] = lst["python_type"]
         if lst["python_type"] == float:
             myPrecisionDict [ n ] = lst["precision"]    #Populating precision dictionary
-        elif lst["python_type"] == datetime.date:
+        elif lst["python_type"] == datetime.date or issubclass(lst['delegate'], delegates.DateDelegate):
+            myFormatDict [ n ] = lst.get('format', local_date_format())         #Populating date Format dictionary
+            myDataTypeDict[ n ] = datetime.date
+        elif lst["python_type"] == datetime.datetime or issubclass(lst['delegate'], delegates.DateTimeDelegate):
+            myDataTypeDict[ n ] = datetime.datetime
             myFormatDict [ n ] = lst["format"]          #Populating date Format dictionary
         elif lst["python_type"] == str:
             if 'length' in lst:
                 myLengthDict [ n ] = lst["length"]          #Populating Column Length dictionary
     ## Writing Data
     row = 3
-    column = 0
     valueAddedInSize = 0
     formatStr = '0'
-    for dictCounter in dataList:                       # iterating the dataList, having dictionary
-        column = 0
+    for data in data_list:                       # iterating the data_list, having dictionary
         cellStyle.num_format_str = '0'
-        for i in range( 0 , len(dictCounter)): #for i in dictCounter:
+        for column, val in enumerate( data ): #for i in dictCounter:
             valueAddedInSize = 0
-            val = dictCounter[i]
             if val != None:
+                # this is to handle fields of type code
+                if isinstance(val, list):
+                    val = '.'.join(val)
                 if not isinstance(val,(str,unicode,int,float,datetime.datetime,datetime.time,datetime.date,
                                        ExcelFormula.Formula) ):
                     val = unicode(val)
@@ -205,9 +234,11 @@ def write_data_to_excel(filename, title, headerList, dataList):
                         for _j in range( 0 , myPrecisionDict[ column ]):
                             formatStr += '0'
                         valueAddedInSize = len(formatStr) # To fit the cell width + 1 (of dot(.))
-                    elif myDataTypeDict[ column ] == datetime.date:
+                    elif myDataTypeDict[ column ] == datetime.date or isinstance(header_delegates[column], delegates.DateDelegate):
                         formatStr = myFormatDict[column]
                         val = datetime.datetime( day = val.day, year = val.year, month = val.month)
+                    elif myDataTypeDict[ column ] == datetime.datetime:
+                        formatStr = myFormatDict[column]
                     elif myDataTypeDict[ column ] == bool:
                         formatStr = '0'
                     else:
@@ -220,18 +251,18 @@ def write_data_to_excel(filename, title, headerList, dataList):
                 bottomleftCellStyle.num_format_str = formatStr
             elif val == None:
                 val = ' '
-            if row - 2  == len(dataList):
+            if row - 2  == len(data_list):
                 #we re at the bottom row
-                if i==0:
+                if column == 0:
                     ws.write(row , column, val , bottomleftCellStyle)
-                elif i  == len(dictCounter)-1:
+                elif column  == number_of_columns - 1:
                     ws.write(row , column, val , bottomrightCellStyle)
                 else:
                     ws.write(row , column, val , bottomCellStyle)
             else:
-                if i==0:
+                if column == 0:
                     ws.write(row , column, val , leftCellStyle)
-                elif  i == len(dictCounter)-1:
+                elif column == number_of_columns - 1:
                     ws.write(row , column, val , rightCellStyle)
                 else:
                     ws.write(row , column, val , cellStyle)
@@ -240,3 +271,4 @@ def write_data_to_excel(filename, title, headerList, dataList):
             column = column + 1
         row = row + 1
     w.save(filename)
+
