@@ -1,21 +1,18 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
 #
 #  This file may be used under the terms of the GNU General Public
 #  License version 2.0 as published by the Free Software Foundation
-#  and appearing in the file LICENSE.GPL included in the packaging of
-#  this file.  Please review the following information to ensure GNU
-#  General Public Licensing requirements will be met:
-#  http://www.trolltech.com/products/qt/opensource.html
+#  and appearing in the file license.txt included in the packaging of
+#  this file.  Please review this information to ensure GNU
+#  General Public Licensing requirements will be met.
 #
 #  If you are unsure which license is appropriate for your use, please
-#  review the following information:
-#  http://www.trolltech.com/products/qt/licensing.html or contact
-#  project-camelot@conceptive.be.
+#  visit www.python-camelot.com or contact project-camelot@conceptive.be
 #
 #  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 #  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -29,11 +26,13 @@
 
 import sqlalchemy.types
 import camelot.types
+from camelot.core.sql import  like_op
+from sqlalchemy.sql.operators import between_op
 import datetime
+import operator
 
 from controls import delegates
 from camelot.core import constants
-from camelot.core.utils import ugettext as _
 from camelot.view.utils import (
     bool_from_string,
     date_from_string,
@@ -42,8 +41,12 @@ from camelot.view.utils import (
     int_from_string,
     float_from_string,
     string_from_string,
+    enumeration_to_string,
+    default_language,
 )
 
+_numerical_operators = (operator.eq, operator.ne, operator.lt, operator.le, operator.gt, operator.ge, between_op)
+_text_operators = (operator.eq, operator.ne, like_op)
 
 _sqlalchemy_to_python_type_ = {
 
@@ -52,7 +55,8 @@ _sqlalchemy_to_python_type_ = {
         'editable': True,
         'nullable': True,
         'delegate': delegates.BoolDelegate,
-        'from_string': bool_from_string
+        'from_string': bool_from_string,
+        'operators' : (operator.eq,),
     },
 
     sqlalchemy.types.BOOLEAN: lambda f: {
@@ -60,7 +64,8 @@ _sqlalchemy_to_python_type_ = {
         'editable': True,
         'nullable': True,
         'delegate': delegates.BoolDelegate,
-        'from_string': bool_from_string
+        'from_string': bool_from_string,
+        'operators' : (operator.eq,),
     },
 
     sqlalchemy.types.Date: lambda f: {
@@ -71,7 +76,8 @@ _sqlalchemy_to_python_type_ = {
         'max': None,
         'nullable': True,
         'delegate': delegates.DateDelegate,
-        'from_string': date_from_string
+        'from_string': date_from_string,
+        'operators' : _numerical_operators,
     },
 
     sqlalchemy.types.Time : lambda f: {
@@ -82,7 +88,8 @@ _sqlalchemy_to_python_type_ = {
             'delegate': delegates.TimeDelegate,
             'format': constants.camelot_time_format,
             'nullable': True,
-            'from_string': time_from_string
+            'from_string': time_from_string,
+            'operators': _numerical_operators,
     },
 
     sqlalchemy.types.DateTime : lambda f: {
@@ -93,18 +100,33 @@ _sqlalchemy_to_python_type_ = {
         'format': constants.camelot_datetime_format,
         'nullable': True,
         'delegate': delegates.DateTimeDelegate,
-        'from_string': datetime_from_string
+        'from_string': datetime_from_string,
+        'operators': _numerical_operators,
     },
 
     sqlalchemy.types.Float: lambda f: {
         'python_type': float,
-        'precision': f.precision,
+        'precision': f.precision if not isinstance(f.precision, tuple) else f.precision[1],
         'editable': True,
         'minimum': constants.camelot_minfloat,
         'maximum': constants.camelot_maxfloat,
         'nullable': True,
         'delegate': delegates.FloatDelegate,
-        'from_string': float_from_string
+        'from_string': float_from_string,
+        'operators': _numerical_operators,
+    },
+    
+    sqlalchemy.types.Numeric: lambda f: {
+        'python_type': float,
+        'precision': f.scale,
+        'editable': True,
+        'minimum': constants.camelot_minfloat,
+        'maximum': constants.camelot_maxfloat,
+        'nullable': True,
+        'delegate': delegates.FloatDelegate,
+        'from_string': float_from_string,
+        'operators': _numerical_operators,
+        'decimal':True,
     },
 
     sqlalchemy.types.Integer: lambda f: {
@@ -116,7 +138,8 @@ _sqlalchemy_to_python_type_ = {
         'delegate': delegates.IntegerDelegate,
         'from_string': int_from_string,
         'to_string': unicode,
-        'widget': 'int'
+        'widget': 'int',
+        'operators': _numerical_operators,
     },
 
     sqlalchemy.types.INT: lambda f: {
@@ -127,7 +150,8 @@ _sqlalchemy_to_python_type_ = {
         'nullable': True,
         'delegate': delegates.IntegerDelegate,
         'from_string': int_from_string,
-        'widget': 'int'
+        'widget': 'int',
+        'operators': _numerical_operators,
     },
 
     sqlalchemy.types.String: lambda f: {
@@ -138,6 +162,7 @@ _sqlalchemy_to_python_type_ = {
         'nullable': True,
         'widget': 'str',
         'from_string': string_from_string,
+        'operators' : _text_operators,
     },
 
     sqlalchemy.types.TEXT: lambda f: {
@@ -148,6 +173,7 @@ _sqlalchemy_to_python_type_ = {
         'nullable': True,
         'widget': 'str',
         'from_string': string_from_string,
+        'operators' : _text_operators,
     },
 
     sqlalchemy.types.Unicode: lambda f: {
@@ -158,6 +184,7 @@ _sqlalchemy_to_python_type_ = {
         'nullable': True,
         'widget': 'str',
         'from_string': string_from_string,
+        'operators' : _text_operators,
     },
 
     camelot.types.Image: lambda f: {
@@ -167,7 +194,8 @@ _sqlalchemy_to_python_type_ = {
         'delegate': delegates.ImageDelegate,
         'storage': f.storage,
         'preview_width': 100,
-        'preview_height': 100
+        'preview_height': 100,
+        'operators' : _text_operators,
     },
 
     camelot.types.Code: lambda f: {
@@ -175,7 +203,9 @@ _sqlalchemy_to_python_type_ = {
         'editable': True,
         'delegate': delegates.CodeDelegate,
         'nullable': True,
-        'parts': f.parts
+        'parts': f.parts,
+        'separator': f.separator,
+        'operators' : _text_operators,
     },
 
     camelot.types.IPAddress: lambda f: {
@@ -184,14 +214,17 @@ _sqlalchemy_to_python_type_ = {
         'nullable': True,
         'parts': f.parts,
         'delegate': delegates.CodeDelegate,
-        'widget': 'code'
+        'widget': 'code',
+        'operators' : _text_operators,
     },
 
     camelot.types.VirtualAddress: lambda f: {
         'python_type': str,
         'editable': True,
         'nullable': True,
-        'delegate': delegates.VirtualAddressDelegate
+        'delegate': delegates.VirtualAddressDelegate,
+        'operators' : _text_operators,
+        'from_string' : lambda str:None,
     },
 
     camelot.types.RichText: lambda f: {
@@ -200,6 +233,7 @@ _sqlalchemy_to_python_type_ = {
         'nullable': True,
         'delegate': delegates.RichTextDelegate,
         'from_string': string_from_string,
+        'operators' : [],
     },
 
     camelot.types.Color: lambda f: {
@@ -207,7 +241,8 @@ _sqlalchemy_to_python_type_ = {
         'python_type': str,
         'editable': True,
         'nullable': True,
-        'widget': 'color'
+        'widget': 'color',
+        'operators' : _text_operators,
     },
 
     camelot.types.Rating: lambda f: {
@@ -216,27 +251,26 @@ _sqlalchemy_to_python_type_ = {
         'nullable': True,
         'python_type': int,
         'widget': 'star',
-        'from_string': int_from_string
+        'from_string': int_from_string,
+        'operators' : _numerical_operators,
     },
 
     camelot.types.Enumeration: lambda f: {
         'delegate': delegates.EnumerationDelegate,
         'python_type': str,
-        'choices': [
-            (v, unicode(_(unicode(v).replace('_', ' '))).capitalize()) 
-            for v in f.choices
-        ],
-        'from_string': lambda s:dict((unicode(_(unicode(v).replace('_', ' '))).capitalize(), v)
-                                     for v in f.choices)[s],
+        'choices': [(v, enumeration_to_string(v)) for v in f.choices],
+        'from_string': lambda s:dict((enumeration_to_string(v), v) for v in f.choices)[s],
+        'minimal_column_width':max(len(enumeration_to_string(v)) for v in f.choices),
         'editable': True,
-        'nullable': False,
+        'nullable': True,
         'widget': 'combobox',
+        'operators' : _numerical_operators,
     },
     
     camelot.types.Language: lambda f: {
-        'delegate': delegates.EnumerationDelegate,
+        'delegate': delegates.LanguageDelegate,
         'python_type': str,
-        'choices': f.choices,
+        'default': default_language,
         'from_string': string_from_string,
         'editable': True,
         'nullable': False,
@@ -247,12 +281,13 @@ _sqlalchemy_to_python_type_ = {
         'python_type': str,
         'editable': True,
         'delegate': delegates.FileDelegate,
-        'storage': f.storage
+        'storage': f.storage,
+        'operators' : _text_operators,
     },
 }
 
 #
-# Generate a restructured text table out of the prevous data structure
+# Generate a restructured text table out of the previous data structure
 #
 
 class DummyField(object):
@@ -261,10 +296,12 @@ class DummyField(object):
         self.parts = ['AAA', '99']
         self.choices = ['planned', 'canceled']
         self.precision = 2
+        self.scale = 2
         self.storage = None
+        self.separator = u'.'
 
-row_separator = '+' + '-'*20 + '+' + '-'*30 + '+' + '-'*70 + '+'
-row_format = """| %-18s | %-28s | %-68s |"""
+row_separator = '+' + '-'*20 + '+' + '-'*70 + '+' + '-'*70 + '+'
+row_format = """| %-18s | %-68s | %-68s |"""
 
 doc = """Field types handled through introspection :
 
@@ -279,12 +316,15 @@ field_types.sort(lambda x, y: cmp(x.__name__, y.__name__))
 for field_type in field_types:
     field_attributes = _sqlalchemy_to_python_type_[field_type](DummyField())
     delegate = field_attributes['delegate']
-    row = row_format%(field_type.__name__, delegate.__name__, '.. image:: ../_static/editors/%s_editable.png'%(delegate.editor.__name__))
+    row = row_format%(field_type.__name__, 
+                      ':ref:`%s <delegate-%s>`'%(delegate.__name__, delegate.__name__), 
+                      '.. image:: ../_static/editors/%s_editable.png'%(delegate.editor.__name__))
     doc += row + """
-  """ + row_separator + """
-  """
+""" + row_separator + """
+"""
 
 doc += """
 """
 
 __doc__ = doc
+
