@@ -1,21 +1,18 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
 #
 #  This file may be used under the terms of the GNU General Public
 #  License version 2.0 as published by the Free Software Foundation
-#  and appearing in the file LICENSE.GPL included in the packaging of
-#  this file.  Please review the following information to ensure GNU
-#  General Public Licensing requirements will be met:
-#  http://www.trolltech.com/products/qt/opensource.html
+#  and appearing in the file license.txt included in the packaging of
+#  this file.  Please review this information to ensure GNU
+#  General Public Licensing requirements will be met.
 #
 #  If you are unsure which license is appropriate for your use, please
-#  review the following information:
-#  http://www.trolltech.com/products/qt/licensing.html or contact
-#  project-camelot@conceptive.be.
+#  visit www.python-camelot.com or contact project-camelot@conceptive.be
 #
 #  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 #  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -25,17 +22,21 @@
 #
 #  ============================================================================
 
+"""Module containing the FIFO cache used in the collection proxy to store
+the data that is passed between the model and the gui thread"""
 
-class fifo(object):
-    """fifo, is the actual cache containing a limited set of copies of row data
-    so the data in fifo, is always immediately accessible to the gui thread,
-    with zero delay as you scroll down the table view, fifo is filled and
+class Fifo(object):
+    """Fifo, is the actual cache containing a limited set of copies of row data
+    so the data in Fifo, is always immediately accessible to the gui thread,
+    with zero delay as you scroll down the table view, Fifo is filled and
     refilled with data queried from the database
     
-    the cache can be queried either by the row number or by the primary key
-    of the object represented by the row data.
+    the cache can be queried either by the row number or by object represented 
+    by the row data.
     """
     def __init__(self, max_entries):
+        """:param max_entries: the maximum entries that will be stored in the
+        cache, if more data is added, the oldest data gets removed"""
         self.max_entries = max_entries
         self.entities = []
         self.data_by_rows = dict()
@@ -45,18 +46,24 @@ class fifo(object):
         return u','.join(unicode(e) for e in self.entities)
     
     def add_data(self, row, entity, value):
+        """The entity might allready be on another row, and this row
+        might allready contain an entity"""
+#        try:
+#            previous_entity = self.get_entity_at_row(row)
+#            self.delete_by_entity(previous_entity)
+#        except KeyError:
+#            pass
         self.delete_by_entity(entity)
         self.data_by_rows[row] = (entity, value)
         self.rows_by_entity[entity] = row
         self.entities.append(entity)
         if len(self.entities)>self.max_entries:
             entity = self.entities.pop(0)
-            row = self.rows_by_entity[entity]
-            del self.data_by_rows[row]
-            del self.rows_by_entity[entity]
+            self.delete_by_entity(entity)
       
     def delete_by_row(self, row):
-        (entity, value) = self.data_by_rows[row]
+        """Remove the data and the reference to the object at row"""
+        (entity, _value_) = self.data_by_rows[row]
         del self.data_by_rows[row]
         del self.rows_by_entity[entity] 
         return row
@@ -78,11 +85,20 @@ class fifo(object):
             pass
         return row    
     
+    def has_data_at_row(self, row):
+        """:return: True if there is data in the cache for the row, False if 
+        there isn't"""
+        return row in self.data_by_rows
+    
     def get_data_at_row(self, row):
+        """:return: the data at row"""
         return self.data_by_rows[row][1]
     
     def get_row_by_entity(self, entity):
+        """:return: the row at which an entity is stored"""
         return self.rows_by_entity[entity]
     
     def get_entity_at_row(self, row):
+        """:return: the entity that is stored at a row"""
         return self.data_by_rows[row][0]
+
