@@ -1,21 +1,18 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
 #
 #  This file may be used under the terms of the GNU General Public
 #  License version 2.0 as published by the Free Software Foundation
-#  and appearing in the file LICENSE.GPL included in the packaging of
-#  this file.  Please review the following information to ensure GNU
-#  General Public Licensing requirements will be met:
-#  http://www.trolltech.com/products/qt/opensource.html
+#  and appearing in the file license.txt included in the packaging of
+#  this file.  Please review this information to ensure GNU
+#  General Public Licensing requirements will be met.
 #
 #  If you are unsure which license is appropriate for your use, please
-#  review the following information:
-#  http://www.trolltech.com/products/qt/licensing.html or contact
-#  project-camelot@conceptive.be.
+#  visit www.python-camelot.com or contact project-camelot@conceptive.be
 #
 #  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 #  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -46,12 +43,25 @@ class Form( object ):
     .. image:: ../_static/form/form.png
     """
 
-    def __init__( self, content, scrollbars = False ):
-        """:param content: a list with the field names and forms to render
+    def __init__( self, content, scrollbars = False, columns = 1  ):
+        """
+        :param content: a list with the field names and forms to render
+        :param columns: the number of columns in which to order the fields.
+
+        eg : with 2 columns, the fields ['street', 'city', 'country'] will
+        be ordered as :
+
+        +-------------+--------------+
+        | street      | city         |
+        +-------------+--------------+
+        | country     |              |
+        +-------------+--------------+
+
         """
         assert isinstance( content, list )
         self._content = content
         self._scrollbars = scrollbars
+        self._columns = columns
 
     def get_fields( self ):
         """:return: the fields, visible in this form"""
@@ -67,7 +77,7 @@ class Form( object ):
                 yield field;
 
 
-    def removeField( self, original_field ):
+    def remove_field( self, original_field ):
         """Remove a field from the form, This function can be used to modify
         inherited forms.
 
@@ -76,7 +86,7 @@ class Form( object ):
         """
         for c in self._content:
             if isinstance( c, Form ):
-                c.removeField( original_field )
+                c.remove_field( original_field )
             if original_field in self._content:
                 self._content.remove( original_field )
                 return True
@@ -85,7 +95,7 @@ class Form( object ):
                 return True
         return False
 
-    def replaceField( self, original_field, new_field ):
+    def replace_field( self, original_field, new_field ):
         """Replace a field on this form with another field.  This function can be used to
         modify inherited forms.
 
@@ -95,7 +105,7 @@ class Form( object ):
         """
         for i, c in enumerate( self._content ):
             if isinstance( c, Form ):
-                c.replaceField( original_field, new_field )
+                c.replace_field( original_field, new_field )
             elif c == original_field:
                 self._content[i] = new_field
                 return True
@@ -107,8 +117,56 @@ class Form( object ):
     def __unicode__( self ):
         return 'Form(%s)' % ( u','.join( unicode( c ) for c in self._content ) )
 
+    def render_ooxml( self, obj, delegates ):
+        """Generator for lines of text in Office Open XML representing this form, using tables
+        :param obj: the object or entity that will be rendered
+        :param delegates: a dictionary mapping field names to their delegate
+        """
+        yield '<w:tbl>'
+        yield '    <w:tblPr>'
+        yield '      <w:tblW w:w="0" w:type="auto"/>'
+        yield '      <w:tblBorders>'
+        yield '          <w:top w:val="single" w:sz="4" wx:bdrwidth="10" w:space="0" w:color="auto"/>'
+        yield '          <w:left w:val="single" w:sz="4" wx:bdrwidth="10" w:space="0" w:color="auto"/>'
+        yield '          <w:bottom w:val="single" w:sz="4" wx:bdrwidth="10" w:space="0" w:color="auto"/>'
+        yield '          <w:right w:val="single" w:sz="4" wx:bdrwidth="10" w:space="0" w:color="auto"/>'
+        yield '          <w:insideH w:val="single" w:sz="4" wx:bdrwidth="10" w:space="0" w:color="auto"/>'
+        yield '          <w:insideV w:val="single" w:sz="4" wx:bdrwidth="10" w:space="0" w:color="auto"/>'
+        yield '        </w:tblBorders>'
+        yield '      <w:tblLook w:val="04A0"/>'
+        yield '    </w:tblPr>'
+        yield '    <w:tblGrid>'
+        yield '      <w:gridCol w:w="4811"/>'
+        yield '      <w:gridCol w:w="4811"/>'
+        yield '    </w:tblGrid>'
+        yield '    <w:tr>'
+        for index, field in enumerate(self._content):
+            if index % self._columns == 0 and index != 0:
+                yield '    </w:tr><w:tr>'
+            yield '<w:tc>'
+            yield '  <w:tcPr>'
+            yield '    <w:tcW w:w="4811" w:type="dxa"/>'
+            yield '    <w:shd w:val="clear" w:color="auto" w:fill="auto"/>'
+            yield '  </w:tcPr>'
+            yield '<w:p>'
+            lines = []
+            if isinstance(field, Label):
+                lines = field.render_ooxml()
+            elif isinstance(field, Form):
+                lines = field.render_ooxml( obj, delegates )
+            elif isinstance(field, basestring):
+                delegate = delegates[field]
+                value = getattr(obj, field)
+                lines = delegate.render_ooxml(value)
+            for line in lines:
+                yield line
+            yield '</w:p>'
+            yield '</w:tc>'
+        yield '</w:tr>'
+        yield '</w:tbl>'
+
     @gui_function
-    def render( self, widgets, parent = None, nomargins = False ):
+    def render( self, widgets, parent = None, nomargins = False):
         """:param widgets: a dictionary mapping each field in this form to a tuple
         of (label, widget editor)
 
@@ -118,52 +176,89 @@ class Form( object ):
         from camelot.view.controls.editors.wideeditor import WideEditor
 
         from PyQt4 import QtGui
-
         form_layout = QtGui.QGridLayout()
-        row = 0
+
+        # where 1 column in the form is a label and a field, so two columns in the grid
+        columns = min(self._columns, len(self._content))
+        # make sure all columns have the same width
+        if columns > 1:
+            for i in range(columns*2):
+                form_layout.setColumnStretch(i, 1)
+
+        row_span = 1
+
+        class cursor(object):
+
+            def __init__(self):
+                self.row = 0
+                self.col = 0
+
+            def next_row(self):
+                self.row = self.row + 1
+                self.col = 0
+
+            def next_col(self):
+                self.col = self.col + 2
+                if self.col >= columns * 2:
+                    self.next_row()
+
+            def next_empty_row(self):
+                if self.col!=0:
+                    self.next_row()
+
+            def __str__(self):
+                return '%s,%s'%(self.row, self.col)
+
+        c = cursor()
+        
+        has_vertical_expanding_row = False
         for field in self._content:
             if isinstance( field, Form ):
-                col = 0
-                row_span = 1
-                col_span = 2
+                has_vertical_expanding_row = True
+                c.next_empty_row()
+                col_span = 2 * columns
                 f = field.render( widgets, parent, True )
                 if isinstance( f, QtGui.QLayout ):
-                    form_layout.addLayout( f, row, col, row_span, col_span )
+                    form_layout.addLayout( f, c.row, c.col, row_span, col_span )
                 else:
-                    form_layout.addWidget( f, row, col, row_span, col_span )
-                row += 1
+                    form_layout.addWidget( f, c.row, c.col, row_span, col_span )
+                c.next_row()
             elif field in widgets:
-                col = 0
-                row_span = 1
                 label, editor = widgets[field]
                 if isinstance( editor, ( WideEditor, ) ):
-                    col_span = 2
+                    c.next_empty_row()
+                    col_span = 2 * columns
                     if label:
-                        form_layout.addWidget( label, row, col, row_span, col_span )
-                        row += 1
-                    form_layout.addWidget( editor, row, col, row_span, col_span )
-                    row += 1
+                        form_layout.addWidget( label, c.row, c.col, row_span, col_span )
+                        c.next_row()
+                    form_layout.addWidget( editor, c.row, c.col, row_span, col_span )
+                    c.next_row()
                 else:
                     col_span = 1
                     if label:
-                        form_layout.addWidget( label, row, col, row_span, col_span )
-                    #form_layout.addWidget(editor, row, col + 1, row_span, col_span, Qt.AlignRight)
-                    form_layout.addWidget( editor, row, col + 1, row_span, col_span )
-                    row += 1
+                        form_layout.addWidget( label, c.row, c.col, row_span, col_span )
+                    form_layout.addWidget( editor, c.row, c.col + 1, row_span, col_span )
+                    c.next_col()
+                size_policy = editor.sizePolicy()
+                if size_policy.verticalPolicy()==QtGui.QSizePolicy.Expanding:
+                    has_vertical_expanding_row = True
+            else:
+                logger.warning('ProgrammingError : widgets should contain a widget for field %s'%unicode(field))
 
         if self._content and form_layout.count():
-            # get last item in the layout
-            last_item = form_layout.itemAt( form_layout.count() - 1 )
-
-            # if last item does not contain a widget, 0 is returned
-            # which is fine with the isinstance test
-            w = last_item.widget()
-
-            # add stretch only if last item is not expandable
-            last_size_policy = w.sizePolicy()
-            if last_size_policy.verticalPolicy()==QtGui.QSizePolicy.Expanding:
-                pass
-            else:
+#            # get last item in the layout
+#            last_item = form_layout.itemAt( form_layout.count() - 1 )
+#
+#            # if last item does not contain a widget, 0 is returned
+#            # which is fine with the isinstance test
+#            w = last_item.widget()
+#
+#            # add stretch only if last item is not expandable
+#            last_size_policy = w.sizePolicy()
+#            if last_size_policy.verticalPolicy()==QtGui.QSizePolicy.Expanding:
+#                pass
+#            else:
+            if not has_vertical_expanding_row:
                 form_layout.setRowStretch( form_layout.rowCount(), 1 )
 
         form_widget = QtGui.QWidget( parent )
@@ -189,20 +284,31 @@ class Form( object ):
 class Label( Form ):
     """Render a label with a QLabel"""
 
-    def __init__( self, label, alignment="left" ):
+    def __init__( self, label, alignment='left', style=None):
         """
         :param label : string to be displayed in the label
         :param alignment : alignment of text in the label. values that make sense 'left', 'right' or 'center'
+        :param style : string of cascading stylesheet instructions
         """
         super( Label, self ).__init__( [] )
         self.label = label
         self.alignment = alignment
+        self.style = style
+
+    def render_ooxml( self ):
+        """Generator for label text in Office Open XML representing this form"""
+        yield '<w:r>'
+        yield '  <w:t>%s</w:t>' % self.label
+        yield '</w:r>'
+
 
     @gui_function
     def render( self, widgets, parent = None, nomargins = False ):
         from PyQt4 import QtGui
-        widget = QtGui.QLabel( '<p align="%s">%s</p>' % (self.alignment, unicode(self.label).capitalize()) )
-
+        if self.style:
+            widget = QtGui.QLabel( '<p align="%s" style="%s">%s</p>' % (self.alignment, self.style,unicode(self.label)) )
+        else:
+            widget = QtGui.QLabel( '<p align="%s">%s</p>' % (self.alignment,unicode(self.label)) )
         return widget
 
 class TabForm( Form ):
@@ -215,11 +321,11 @@ class TabForm( Form ):
   .. image:: ../_static/form/tab_form.png
     """
 
-    NORTH = 'North' 
+    NORTH = 'North'
     SOUTH = 'South'
     WEST = 'West'
     EAST = 'East'
-    
+
     def __init__( self, tabs, position=NORTH ):
         """
         :param tabs: a list of tuples of (tab_label, tab_form)
@@ -269,11 +375,24 @@ class TabForm( Form ):
             if label == tab_label:
                 return form
 
-    def replaceField( self, original_field, new_field ):
+    def replace_field( self, original_field, new_field ):
+        super(TabForm, self).replace_field( original_field, new_field )
         for _label, form in self.tabs:
-            if form.replaceField( original_field, new_field ):
+            if form.replace_field( original_field, new_field ):
                 return True
         return False
+    
+    def remove_field( self, original_field ):
+        super(TabForm, self).remove_field( original_field )
+        for _label, form in self.tabs:
+            if form.remove_field( original_field ):
+                return True
+        return False
+    
+    def _get_fields_from_form( self ):
+        for _label, form in self.tabs:
+            for field in form._get_fields_from_form():
+                yield field
 
     @gui_function
     def render( self, widgets, parent = None, nomargins = False ):
@@ -281,8 +400,15 @@ class TabForm( Form ):
         from PyQt4 import QtGui
         widget = QtGui.QTabWidget( parent )
         widget.setTabPosition(getattr(QtGui.QTabWidget, self.position))
+        vertical_expanding = False
         for tab_label, tab_form in self.tabs:
-            widget.addTab( tab_form.render( widgets, widget ), unicode(tab_label).capitalize() )
+            tab_form_widget = tab_form.render( widgets, widget )
+            size_policy = tab_form_widget.sizePolicy()
+            if size_policy.verticalPolicy()==QtGui.QSizePolicy.Expanding:
+                vertical_expanding = True
+            widget.addTab( tab_form_widget, unicode(tab_label) )
+        if vertical_expanding:
+            widget.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
         return widget
 
 
@@ -296,23 +422,28 @@ class HBoxForm( Form ):
 
   """
 
-    def __init__( self, columns ):
+    def __init__( self, columns, scrollbars=False ):
         """:param columns: a list of forms to display in the different columns
         of the horizontal box"""
         assert isinstance( columns, list )
         self.columns = [structure_to_form( col ) for col in columns]
         super( HBoxForm, self ).__init__( sum( ( column_form.get_fields()
-                                      for column_form in self.columns ), [] ) )
+                                      for column_form in self.columns ), [] ), scrollbars=scrollbars )
 
     def __unicode__( self ):
         return 'HBoxForm [ %s\n         ]' % ( '         \n'.join( [unicode( form ) for form in self.columns] ) )
 
-    def replaceField( self, original_field, new_field ):
+    def replace_field( self, original_field, new_field ):
         for form in self.columns:
-            if form.replaceField( original_field, new_field ):
+            if form.replace_field( original_field, new_field ):
                 return True
         return False
 
+    def _get_fields_from_form( self ):
+        for form in self.columns:
+            for field in form._get_fields_from_form():
+                yield field
+
     @gui_function
     def render( self, widgets, parent = None, nomargins = False ):
         logger.debug( 'rendering %s' % self.__class__.__name__ )
@@ -345,12 +476,17 @@ class VBoxForm( Form ):
         self.rows = [structure_to_form( row ) for row in rows]
         super( VBoxForm, self ).__init__( sum( ( row_form.get_fields() for row_form in self.rows ), [] ) )
 
-    def replaceField( self, original_field, new_field ):
+    def replace_field( self, original_field, new_field ):
         for form in self.rows:
-            if form.replaceField( original_field, new_field ):
+            if form.replace_field( original_field, new_field ):
                 return True
         return False
 
+    def _get_fields_from_form( self ):
+        for form in self.rows:
+            for field in form._get_fields_from_form():
+                yield field
+
     def __unicode__( self ):
         return 'VBoxForm [ %s\n         ]' % ( '         \n'.join( [unicode( form ) for form in self.rows] ) )
 
@@ -369,6 +505,12 @@ class VBoxForm( Form ):
         widget.setLayout( form_layout )
         return widget
 
+class ColumnSpan( Form ):
+    def __init__(self, field=None, num=2):
+        self.num = num
+        self.field = field
+        super( ColumnSpan, self ).__init__( [field] )
+
 class GridForm( Form ):
     """Put different fields into a grid, without a label.  Row or column labels can be added
   using the Label form::
@@ -378,11 +520,12 @@ class GridForm( Form ):
   .. image:: ../_static/form/grid_form.png
   """
 
-    def __init__( self, grid ):
+    def __init__( self, grid, nomargins = False ):
         """:param grid: A list for each row in the grid, containing a list with all fields that should be put in that row
         """
         assert isinstance( grid, list )
         self._grid = grid
+        self._nomargins = nomargins
         fields = []
         for row in grid:
             assert isinstance( row, list )
@@ -410,13 +553,25 @@ class GridForm( Form ):
         widget = QtGui.QWidget( parent )
         grid_layout = QtGui.QGridLayout()
         for i, row in enumerate( self._grid ):
+            skip = 0
             for j, field in enumerate( row ):
+                num = 1
+                if isinstance( field, ColumnSpan ):
+                    num = field.num
+                    field = field.field
+
                 if isinstance( field, Form ):
-                    grid_layout.addWidget( field.render( [], grid_layout ), i, j )
+                    grid_layout.addWidget( field.render( widgets, parent ), i, j + skip, 1, num )
+                    skip += num - 1
                 else:
                     _label, editor = widgets[field]
-                    grid_layout.addWidget( editor, i, j )
+                    grid_layout.addWidget( editor, i, j + skip, 1, num )
+                    skip += num - 1
+
         widget.setLayout( grid_layout )
+        if nomargins:
+            grid_layout.setContentsMargins( 0, 0, 0, 0 )
+
         return widget
 
 class WidgetOnlyForm( Form ):
@@ -442,15 +597,21 @@ class GroupBoxForm( Form ):
   .. image:: ../_static/form/group_box_form.png
   """
 
-    def __init__( self, title, content ):
+    def __init__( self, title, content, scrollbars=None, min_width=None, min_height=None, columns=1 ):
         self.title = title
-        Form.__init__( self, content )
+        self.min_width = min_width
+        self.min_height = min_height
+        if isinstance(content, Form):
+            content = [content]
+        Form.__init__( self, content, scrollbars, columns=columns )
 
     @gui_function
     def render( self, widgets, parent = None, nomargins = False ):
         from PyQt4 import QtGui
         widget = QtGui.QGroupBox( unicode(self.title), parent )
         layout = QtGui.QVBoxLayout()
+        if self.min_width and self.min_height:
+            widget.setMinimumSize ( self.min_width, self.min_height )
         widget.setLayout( layout )
         form = Form.render( self, widgets, widget, nomargins )
         layout.addWidget( form )
@@ -468,3 +629,4 @@ def structure_to_form( structure ):
     if isinstance( structure, Form ):
         return structure
     return Form( structure )
+
