@@ -1,19 +1,45 @@
+#  ============================================================================
+#
+#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  www.conceptive.be / project-camelot@conceptive.be
+#
+#  This file is part of the Camelot Library.
+#
+#  This file may be used under the terms of the GNU General Public
+#  License version 2.0 as published by the Free Software Foundation
+#  and appearing in the file license.txt included in the packaging of
+#  this file.  Please review this information to ensure GNU
+#  General Public Licensing requirements will be met.
+#
+#  If you are unsure which license is appropriate for your use, please
+#  visit www.python-camelot.com or contact project-camelot@conceptive.be
+#
+#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
+#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+#
+#  For use of this library in commercial applications, please contact
+#  project-camelot@conceptive.be
+#
+#  ============================================================================
 '''
 Created on Sep 9, 2009
 
 @author: tw55413
 '''
 import logging
+import sys
 logger = logging.getLogger('camelot.view.model_thread.signal_slot_model_thread')
 
 from PyQt4 import QtCore
 
-from camelot.view.model_thread import AbstractModelThread, gui_function, model_function, setup_model
+from camelot.view.model_thread import AbstractModelThread, gui_function, setup_model
+from camelot.core.threading import synchronized
+from camelot.view.controls.exception import register_exception
 
 class Task(QtCore.QObject):
 
-    finished = QtCore.SIGNAL('finished')
-    exception = QtCore.SIGNAL('exception')
+    finished = QtCore.pyqtSignal(object)
+    exception = QtCore.pyqtSignal(object)
 
     def __init__(self, request, name=''):
         QtCore.QObject.__init__(self)
@@ -29,31 +55,23 @@ class Task(QtCore.QObject):
         logger.debug('executing %s' % (self._name))
         try:
             result = self._request()
-            #import sip
-            #print '===', self._name, '==='
-            #print self.receivers(QtCore.SIGNAL('finished'))
-            #sip.dump(self)
-            self.emit(QtCore.SIGNAL('finished'), result )
+            self.finished.emit( result )
         except Exception, e:
-            logger.error( 'exception caught in model thread', exc_info = e )
-            exception_info = (e, '')
-            self.emit(QtCore.SIGNAL('exception'), exception_info)
-
-def synchronized( original_function ):
-    """Decorator for synchronized access to an object, the object should
-    have an attribute _mutex which is of type QMutex
-    """
-
-    from functools import wraps
-
-    @wraps( original_function )
-    def wrapper(self, *args, **kwargs):
-        QtCore.QMutexLocker(self._mutex)
-        return original_function(self, *args, **kwargs)
-
-    return wrapper
+            name, trace = register_exception(logger, 'exception caught in model thread while executing %s'%self._name, e)
+            self.exception.emit( (name, trace) )
+            # the stack might contain references to QT objects which could be kept alive this way
+            sys.exc_clear()
+        except:
+            logger.error( 'unhandled exception in model thread' )
+            sys.exc_clear()
 
 class TaskHandler(QtCore.QObject):
+    """A task handler is an object that handles tasks that appear in a queue,
+    when its handle_task method is called, it will sequentially handle all tasks
+    that are in the queue.
+    """
+
+    task_handler_busy_signal = QtCore.pyqtSignal(bool)
 
     def __init__(self, queue):
         """:param queue: the queue from which to pop a task when handle_task
@@ -63,17 +81,17 @@ class TaskHandler(QtCore.QObject):
         self._queue = queue
         self._tasks_done = []
         self._busy = False
-        #logger.debug("TaskHandler created.")
+        logger.debug("TaskHandler created.")
 
-    @synchronized
     def busy(self):
         """:return True/False: indicating if this task handler is busy"""
         return self._busy
 
-    @synchronized
+    @QtCore.pyqtSlot()
     def handle_task(self):
+        """Handle all tasks that are in the queue"""
         self._busy = True
-        self.emit( AbstractModelThread.thread_busy_signal, True  )
+        self.task_handler_busy_signal.emit( True )
         task = self._queue.pop()
         while task:
             task.execute()
@@ -85,15 +103,18 @@ class TaskHandler(QtCore.QObject):
             task.clear()
             self._tasks_done.append(task)
             task = self._queue.pop()
-        self.emit( AbstractModelThread.thread_busy_signal, False  )
+        self.task_handler_busy_signal.emit( False )
         self._busy = False
 
-class SignalSlotModelThread( QtCore.QThread, AbstractModelThread ):
+class SignalSlotModelThread( AbstractModelThread ):
     """A model thread implementation that uses signals and slots
     to communicate between the model thread and the gui thread
+
+    there is no explicit model thread verification on these methods,
+    since this model thread might not be THE model thread.
     """
 
-    task_available = QtCore.SIGNAL('task_available')
+    task_available = QtCore.pyqtSignal()
 
     def __init__( self, setup_thread = setup_model ):
         """
@@ -101,38 +122,41 @@ class SignalSlotModelThread( QtCore.QThread, AbstractModelThread ):
         everything, by default this will setup the model.  set to None if nothing should
         be done.
         """
-        QtCore.QThread.__init__( self )
-        AbstractModelThread.__init__( self, setup_thread )
+        super(SignalSlotModelThread, self).__init__( setup_thread )
         self._task_handler = None
         self._mutex = QtCore.QMutex()
         self._request_queue = []
         self._connected = False
+        self._setup_busy = True
 
     def run( self ):
         self.logger.debug( 'model thread started' )
         self._task_handler = TaskHandler(self)
-        self.connect(self._task_handler, self.thread_busy_signal, self._thread_busy, QtCore.Qt.QueuedConnection)
+        self._task_handler.task_handler_busy_signal.connect(self._thread_busy, QtCore.Qt.QueuedConnection)
         self._thread_busy(True)
         try:
             self._setup_thread()
         except Exception, e:
-            self.logger.error('thread setup incomplete', exc_info=e)
+            name, trace = register_exception(logger, 'Exception when setting up the SignalSlotModelThread', e)
+            self.setup_exception_signal.emit(name, trace)
         self._thread_busy(False)
         self.logger.debug('thread setup finished')
         # Some tasks might have been posted before the signals were connected to the task handler,
         # so once force the handling of tasks
         self._task_handler.handle_task()
+        self._setup_busy = False
         self.exec_()
         self.logger.debug('model thread stopped')
 
+    @QtCore.pyqtSlot( bool )
     def _thread_busy(self, busy_state):
-        self.emit(self.thread_busy_signal, busy_state)
-                
+        self.thread_busy_signal.emit( busy_state )
+
     @synchronized
     def post( self, request, response = None, exception = None ):
         if not self._connected and self._task_handler:
             # creating this connection in the model thread throws QT exceptions
-            self.connect(self, self.task_available, self._task_handler.handle_task, QtCore.Qt.QueuedConnection)
+            self.task_available.connect( self._task_handler.handle_task, QtCore.Qt.QueuedConnection )
             self._connected = True
         # response should be a slot method of a QObject
         if response:
@@ -142,19 +166,18 @@ class SignalSlotModelThread( QtCore.QThread, AbstractModelThread ):
         task = Task(request, name=name)
         # QObject::connect is a thread safe function
         if response:
-            assert response.im_self
+            assert response.im_self != None
             assert isinstance(response.im_self, QtCore.QObject)
-            self.connect(task, QtCore.SIGNAL('finished'), response, QtCore.Qt.QueuedConnection)
+            task.finished.connect( response, QtCore.Qt.QueuedConnection )
         if exception:
-            self.connect(task, QtCore.SIGNAL('exception'), exception, QtCore.Qt.QueuedConnection)
+            task.exception.connect( exception, QtCore.Qt.QueuedConnection )
         task.moveToThread(self)
-        # only put the task in the queue when it is completely set up 
+        # only put the task in the queue when it is completely set up
         self._request_queue.append(task)
         #print 'task created --->', id(task)
-        self.emit(self.task_available)
+        self.task_available.emit()
 
     @synchronized
-    @model_function
     def pop( self ):
         """Pop a task from the queue, return None if the queue is empty"""
         if len(self._request_queue):
@@ -162,7 +185,6 @@ class SignalSlotModelThread( QtCore.QThread, AbstractModelThread ):
             return task
 
     @synchronized
-    @gui_function
     def busy( self ):
         """Return True or False indicating wether either the model or the
         gui thread is doing something"""
@@ -170,7 +192,7 @@ class SignalSlotModelThread( QtCore.QThread, AbstractModelThread ):
             import time
             time.sleep(1)
         app = QtCore.QCoreApplication.instance()
-        return app.hasPendingEvents() or len(self._request_queue) or self._task_handler.busy()
+        return app.hasPendingEvents() or len(self._request_queue) or self._task_handler.busy() or self._setup_busy
 
     @gui_function
     def wait_on_work(self):
@@ -188,3 +210,4 @@ class SignalSlotModelThread( QtCore.QThread, AbstractModelThread ):
 #      if app.hasPendingEvents():
 #        app.processEvents()
 #        waiting = True
+
