@@ -1,21 +1,18 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
 #
 #  This file may be used under the terms of the GNU General Public
 #  License version 2.0 as published by the Free Software Foundation
-#  and appearing in the file LICENSE.GPL included in the packaging of
-#  this file.  Please review the following information to ensure GNU
-#  General Public Licensing requirements will be met:
-#  http://www.trolltech.com/products/qt/opensource.html
+#  and appearing in the file license.txt included in the packaging of
+#  this file.  Please review this information to ensure GNU
+#  General Public Licensing requirements will be met.
 #
 #  If you are unsure which license is appropriate for your use, please
-#  review the following information:
-#  http://www.trolltech.com/products/qt/licensing.html or contact
-#  project-camelot@conceptive.be.
+#  visit www.python-camelot.com or contact project-camelot@conceptive.be
 #
 #  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 #  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -25,13 +22,14 @@
 #
 #  ============================================================================
 
-from PyQt4 import QtGui, QtDesigner
+from PyQt4 import QtDesigner
 from camelot.view.plugins import CamelotEditorPlugin
 
 class ManyToOneEditorPlugin(QtDesigner.QPyDesignerCustomWidgetPlugin, CamelotEditorPlugin):
-    
-  def __init__(self, parent = None):
-    QtDesigner.QPyDesignerCustomWidgetPlugin.__init__(self)
-    from camelot.view.controls.editors import Many2OneEditor
-    CamelotEditorPlugin.__init__(self)
-    self._widget = Many2OneEditor
\ No newline at end of file
+
+    def __init__(self, parent = None):
+        QtDesigner.QPyDesignerCustomWidgetPlugin.__init__(self)
+        from camelot.view.controls.editors import Many2OneEditor
+        CamelotEditorPlugin.__init__(self)
+        self._widget = Many2OneEditor
+
