@@ -1,21 +1,18 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
 #
 #  This file may be used under the terms of the GNU General Public
 #  License version 2.0 as published by the Free Software Foundation
-#  and appearing in the file LICENSE.GPL included in the packaging of
-#  this file.  Please review the following information to ensure GNU
-#  General Public Licensing requirements will be met:
-#  http://www.trolltech.com/products/qt/opensource.html
+#  and appearing in the file license.txt included in the packaging of
+#  this file.  Please review this information to ensure GNU
+#  General Public Licensing requirements will be met.
 #
 #  If you are unsure which license is appropriate for your use, please
-#  review the following information:
-#  http://www.trolltech.com/products/qt/licensing.html or contact
-#  project-camelot@conceptive.be.
+#  visit www.python-camelot.com or contact project-camelot@conceptive.be
 #
 #  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 #  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -54,6 +51,10 @@ def local_date_format():
         _local_date_format = unicode(u''.join(format_sequence))
     return _local_date_format
 
+def default_language():
+    locale = QtCore.QLocale()
+    return unicode(locale.name())
+
 class ParsingError(Exception): pass
 
 def string_from_string(s):
@@ -166,7 +167,7 @@ def pyvalue_from_string(pytype, s):
         return int_from_string(s)
 
 def enumeration_to_string(value):
-    return ugettext(unicode(value).replace('_', ' ').capitalize())
+    return ugettext(unicode(value or u'').replace('_', ' ').capitalize())
 
 operator_names = {
     operator.eq : u'=',
@@ -178,3 +179,27 @@ operator_names = {
     like_op : u'like',
     between_op: u'between',
 }
+
+def text_from_richtext(unstripped_text, newlines=True, word_xml=False):
+    # TODO improve/expand
+    from HTMLParser import HTMLParser    
+    strings = []
+    if not unstripped_text:
+	    return ''
+    class HtmlToTextParser(HTMLParser):
+        def handle_data(self, data):
+            from xml.sax.saxutils import escape
+            data = data.strip()
+            if data:
+                newline = ' '
+                if newlines:
+                    newline = "\n"
+                    if word_xml:
+                        newline = '<w:br />'
+                strings.append( '%s%s' % ( escape(data), newline ) )
+
+    parser = HtmlToTextParser()
+    parser.feed(unstripped_text.strip())
+    
+    return ''.join(strings)
+
