@@ -4,14 +4,13 @@ logging.basicConfig(level=logging.INFO, format='[%(levelname)-7s] [%(name)-35s]
 #logging.getLogger('sqlalchemy.engine').setLevel(logging.INFO)
 
 def setup_model():
-  from camelot.model import *
-  from camelot.model.memento import *
-  from camelot.model.synchronization import *
-  from camelot.model.authentication import *
-  from camelot.model.i18n import *
+  import camelot.model
+  import example.model
+  from elixir import setup_all
+  from camelot.model.authentication import updateLastLogin
   setup_all(create_tables=True)
   updateLastLogin()
 
 CAMELOT_MEDIA_ROOT = 'media'
 
-ENGINE = lambda:'sqlite:///'
\ No newline at end of file
+ENGINE = lambda:'sqlite:///'
